/*
  LoRa Communication for Arduino Due
*/

#include <eWBMLoRa.h>
eWBMLoRaModem modem;
USARTClass* SerialShow;

#define TEST_PREDEFINED_LORA_VALUES
//#define TEST_HEX_CODE

String appEui;	// Application Eui
String appKey;	// Application Key
String devAddr; // Device Address
String nwkSKey; // Network Session Key
String appSKey; // Application Session Key

void SetKeyABP()
{
#ifdef TEST_PREDEFINED_LORA_VALUES
	devAddr = "9A:10:04:26";
	nwkSKey = "00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:08";
	appSKey = "00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:09";
	Serial1.print("Device Address ");
	Serial1.println(devAddr);
	Serial1.print("NWS KEY ");
	Serial1.println(nwkSKey);
	Serial1.print("APP SKEY ");
	Serial1.println(appSKey);
#else
	Serial1.println("Enter your Device Address");	// ex) 01:01:01:01
	while (!Serial1.available());
	devAddr = Serial1.readStringUntil('\n');

	Serial1.println("Enter your NWS KEY");			// ex) 00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:08
	while (!Serial1.available());
	nwkSKey = Serial1.readStringUntil('\n');

	Serial1.println("Enter your APP SKEY");			// ex) 00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:09
	while (!Serial1.available());
	appSKey = Serial1.readStringUntil('\n');
#endif
}

void SetKeyOTA()
{
#ifdef TEST_PREDEFINED_LORA_VALUES
	//appEui = "01:01:01:01:01:01:01:01";
	//appKey = "00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:01";
	appEui = "70:B3:D5:7E:D0:01:16:0D";
	appKey = "01:01:01:01:01:01:01:01:01:01:01:01:01:01:01:01";
	Serial1.print("APP EUI ");
	Serial1.println(appEui);
	Serial1.print("APP KEY ");
	Serial1.println(appKey);
#else
	Serial1.println("Enter your APP EUI");			// ex) 01:01:01:01:01:01:01:01
	while (!Serial1.available());
	appEui = Serial1.readStringUntil('\n');

	Serial1.println("Enter your APP KEY");			// ex) 00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:07
	while (!Serial1.available());
	appKey = Serial1.readStringUntil('\n');
#endif
}

void TestSend()
{
	int ret;
	// Send Port : 3, Send Data : "TEST LORA"
#ifdef TEST_HEX_CODE
	ret = modem.SendPacket(3, "54455354204c4f5241", true); // ASCII "TEST LORA" TO HEX "0x54 0x45 0x53 0x54 0x20 0x4c 0x4f 0x52 0x41"
#else
	ret = modem.SendPacket(3, "TEST LORA", false);
#endif
	if (ret > 0)
		SerialShow->println("SEND SUCCESS");
	else
		SerialShow->println("SEND FAIL");
}

void setup() 
{
	SetDebug(&Serial1);
	SerialShow = &Serial1; // Select Console Serial Port : Serial1 or Serial2 or Serial3
	modem.ResetHW();
	SerialShow->begin(115200);
	Serial.begin(115200);
	while (!Serial);
	
	SerialShow->println("LoRa Network Start");
	if (!modem.begin()) 
	{
		SerialShow->println("Failed to start module");
		while (1) {}
	}

	SerialShow->println("LoRa Module Version : " + modem.Get_AT_VER());
	SerialShow->println("LoRa Device EUI : " + modem.Get_AT_DEVEUI());

	int LoRaMode = 0;
	while (LoRaMode != 1 && LoRaMode != 2) 
	{
		SerialShow->println("Select Lora Mode : OTAA (1) or ABP (2) ?");
		while (!SerialShow->available());
		LoRaMode = SerialShow->readStringUntil('\n').toInt();
	}
	
	int connected;
	if (LoRaMode == 1) 
	{
		SetKeyOTA(); 
		connected = modem.joinOTAA(appEui, appKey);
	}
	else if (LoRaMode == 2) 
	{
		SetKeyABP();
		connected = modem.joinABP(devAddr, nwkSKey, appSKey);
	}

	if (connected)
	{
		SerialShow->println("JOINED");
	}
	else
	{
		SerialShow->println("Fail : Check the gateway and network server.");
		while (1) {}
	}
	delay(5000);
}

void loop() 
{
	TestSend();

	String strRecved;
	if(modem.IsRecved(strRecved))
		SerialShow->println("Received Packet [" + strRecved + "]\r\n");	// Received Packet [Port:Data]
}